﻿/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Text;
using System.Collections.Generic;
using LogMeIn.GoToCoreLib.Api.Common;
using LogMeIn.GoToCoreLib.Api.Model;

namespace LogMeIn.GoToCoreLib.Api
{
    /// <summary>
    /// Class containing methods for OAuth2 authentication.
    /// </summary>
    public class OAuth2Api
    {
        private string basePath = "https://api.getgo.com/oauth/v2";
        private readonly ApiInvoker apiInvoker = ApiInvoker.GetInstance();

        /// <summary>
        /// Initializes a new instance of the OAuth2Api class using the
        /// default endpoint base url for the services being accessed.
        /// </summary>
        /// <param name="consumerKey">The app's consumer key (client ID).</param>
        /// <param name="consumerSecret">The app's consumer secret (client secret).</param>
        public OAuth2Api(String consumerKey, String consumerSecret)
        {
            byte[] credentialsAsBytes = Encoding.UTF8.GetBytes(consumerKey + ":" + consumerSecret);
            apiInvoker.AddDefaultHeader("Authorization", "Basic " + Convert.ToBase64String(credentialsAsBytes));
        }

        /// <summary>
        /// Initializes a new instance of the OAuth2Api class using an endpoint base
        /// url other than the default. Use this constructor only if you should need
        /// to override the default endpoint base url.
        /// </summary>
        /// <param name="basePath">The endpoint base url for the services being accessed.</param>
        /// <param name="consumerKey">The app's consumer key (client ID).</param>
        /// <param name="consumerSecret">The app's consumer secret (client secret).</param>
        public OAuth2Api(String basePath, String consumerKey, String consumerSecret)
            : this(consumerKey, consumerSecret)
        {
            this.basePath = basePath;
        }

        /// <summary>
        /// Gets the API invoker
        /// </summary>
        public ApiInvoker Invoker
        {
            get { return apiInvoker; }
        }

        /// <summary>
        /// Gets the endpoint base url for the services being accessed
        /// </summary>
        public String BasePath
        {
            get { return basePath; }
        }

        /// <summary>
        /// Returns the authorization URL where the browser will be directed givent the consumer key
        /// of the app and a redirect Url.
        /// </summary>
        /// <param name="clientId">The consumer (API) key of the application.</param>
        /// <returns>The authorization URL where the browser will be directed.</returns>
        public string GetOAuth2AuthorisationUrl(string clientId)
        {
            var authUrl = string.Format(basePath + "/authorize?client_id={0}&response_type=code", clientId);
            return authUrl;
        }

        /// <summary>
        /// Gets the Response Key from the OAuth flow contained in the user redirection back to the application.
        /// This must be exchanged for an Access Token.
        /// </summary>
        /// <param name="navigatedUrl">The user redirection.</param>
        /// <returns>The response key.</returns>
        public string GetResponseKey(Uri navigatedUrl)
        {
            if (string.IsNullOrEmpty(navigatedUrl.Query))
                return null;

            var pairs = navigatedUrl.Query.Split(new[] { '&', '?' }, StringSplitOptions.RemoveEmptyEntries);
            var parameters = new Dictionary<string, string>();

            foreach (string pair in pairs)
            {
                var nameValue = pair.Split(new[] { '=' });
                parameters[nameValue[0]] = nameValue[1];
            }

            if (parameters.ContainsKey("code"))
                return parameters["code"];

            if (parameters.ContainsKey("error_description"))
                throw new ApiException(Uri.UnescapeDataString(parameters["error_description"]));

            return null;
        }

        /// <summary>
        /// Exchanges the response key for an access token.
        /// </summary>
        /// <param name="responseKey">The response key.</param>
        /// <returns>Information containing the access token and other details about the user's product account.</returns>
        public TokenResponse GetAccessTokenResponse(string responseKey)
        {
            var formParams = new Dictionary<String, object>
                                 {
                                     {"grant_type", "authorization_code"},
                                     {"code", responseKey}
                                 };

            return ExecuteTokenRequest(formParams);
        }

        /// <summary>
        /// Direct Login
        /// Generates an access token from user credentials.
        /// </summary>
        /// <param name="username">The GoTo product user email.</param>
        /// <param name="password">The GoTo product user password.</param>
        /// <returns>Information containing the access token and other details about the user's product account.</returns>
        /// <exception cref="ApiException">If an error occurred during the access token generation.</exception>
        [Obsolete("This authentication API is now deprecated. All new clients will not be able to use this API. If you have a client for which the direct login works, that will continue to work for now.")]
        public TokenResponse DirectLogin(string username, string password)
        {
            if (username == null)
                throw new ApiException("Required parameter username is null.");

            if (password == null)
                throw new ApiException("Required parameter password is null.");

            var formParams = new Dictionary<String, object>();
            formParams.Add("grant_type", "password");
            formParams.Add("username", username);
            formParams.Add("password", password);

            return ExecuteTokenRequest(formParams);
        }

        /// <summary>
        /// Exchanges the refresh token for an access token.
        /// </summary>
        /// <param name="refreshToken">The refresh token from the access token response.</param>
        /// <returns>The refresh token from the access token response.</returns>
        /// <exception cref="ApiException">If an error occurred during exchanging the refresh token for an access token.</exception>
        public TokenResponse GetAccessTokenUsingRefreshToken(string refreshToken)
        {
            if (refreshToken == null)
                throw new ApiException("Required parameter refreshToken is null.");

            var formParams = new Dictionary<String, object>();
            formParams.Add("grant_type", "refresh_token");
            formParams.Add("refresh_token", refreshToken);

            return ExecuteTokenRequest(formParams);
        }

        private TokenResponse ExecuteTokenRequest(Dictionary<string, object> formParams)
        {
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var response = apiInvoker.InvokeAPI(basePath, "/token", "POST", queryParams, null, headerParams,
                formParams, typeof(TokenResponse), "application/x-www-form-urlencoded");
            if (response != null)
                return (TokenResponse)response;
            return null;
        }
    }
}
